/**
 *******************************************************************************
 * @file    main.c
 *
 * @brief   Main program body.
 *
 * @author  DPM
 *
 * @version V1.0.0
 *
 * @date    2024-5-6
 *
 *******************************************************************************/

/*
    Copyright (c) 2024, DP Microelectronics Inc.

    Redistribution and use in source and binary forms, with or without modification,
are permitted provided that the following conditions are met:

    1. Redistributions of source code must retain the above copyright notice, this
       list of conditions and the following disclaimer.
    2. Redistributions in binary form must reproduce the above copyright notice,
       this list of conditions and the following disclaimer in the documentation
       and/or other materials provided with the distribution.
    3. Neither the name of the copyright holder nor the names of its contributors
       may be used to endorse or promote products derived from this software without
       specific prior written permission.

    THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS "AS IS"
AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED
WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE ARE DISCLAIMED.
IN NO EVENT SHALL THE COPYRIGHT HOLDER OR CONTRIBUTORS BE LIABLE FOR ANY DIRECT,
INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT
NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR
PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY,
WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE)
ARISING IN ANY WAY OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY
OF SUCH DAMAGE.
*/

/* Files include */
#include "main.h"
#include "dpm32m0xx.h"
#include "dpm32m0xx_eval.h"
#include "systick.h"
#include <stdio.h>

/**
  * @addtogroup DPM32M08X_LibSamples
  * @{
  */

/**
  * @addtogroup UART
  * @{
  */

/**
  * @addtogroup UART_HalfDuplex
  * @{
  */

/* Private typedef --------------------------------------------------------------*/

/* Private define ---------------------------------------------------------------*/

/* Private macro ----------------------------------------------------------------*/

/* Private variables ------------------------------------------------------------*/

/* Private functions ------------------------------------------------------------*/

/**
  * @brief  The program entry function.
  * @retval int
  */
int main(void)
{
  /* Configure the system clock */
  SystemClockInit();

  /* Configure the system tick */
  SystickInit();

  /* LED Init */
  EVAL_LedInit(LED1);

  /* Print Uart Init */
  EVAL_ComInit();

  /* HalfDuplex Uart Init */
  /* enable GPIO clock */
  RCC_Unlock();
  RCC_AHBPeriphClockCmd(RCC_AHB_PERIPH_GPIO, ENABLE);
  RCC_APBPeriphClockCmd(RCC_APB_PERIPH_UART1, ENABLE);
  RCC_Lock();

  /* connect port to UARTx Pin */
  GPIO_InitTypeStruct GPIO_InitStruct;
  GPIO_StructInit(&GPIO_InitStruct);
  GPIO_InitStruct.GPIO_Mode = GPIO_MODE_ALT_FUNC;
  GPIO_InitStruct.GPIO_Speed = GPIO_SPEED_HIGH;
  GPIO_InitStruct.GPIO_OType =  GPIO_OTYPE_OD;
  GPIO_InitStruct.GPIO_PuPd = GPIO_PUPD_UP;

  GPIO_InitStruct.GPIO_Pin =  GPIO_PIN_0;
  GPIO_Init(GPIOD, &GPIO_InitStruct);
  GPIO_AltFuncConfig(GPIOD, GPIO_PIN_0, GPIO_ALT_FUNC_0);

  /* UART configure */
  UART_DeInit(UART1);
  UART_InitTypeStruct UART_InitStruct;
  UART_StructInit(&UART_InitStruct);
  UART_InitStruct.UART_BaudRate = 115200;
  UART_InitStruct.UART_DataLen = UART_DATA_LEN_8;
  UART_InitStruct.UART_StopBits = UART_STOP_BITS_1;
  UART_InitStruct.UART_Parity = UART_PARITY_NO;
  UART_InitStruct.UART_Mode = UART_MODE_RX_TX;
  UART_Init(UART1, &UART_InitStruct);
  UART_Cmd(UART1, ENABLE);
  UART_HalfDuplexCmd(UART1, ENABLE);

  uint32_t ledTime = 0;
  uint8_t data = 0;
  printf("UART1 halfdeplex example\r\n");
  while(1)
  {
    if(UART_GetIntFlagStatus(UART1, UART_INT_FLAG_RXNE) == SET)
    {
      data = UART_ReceiveData(UART1);
      UART_SendData(UART1, data);
      printf("%d", data);

      while(UART_GetIntFlagStatus(UART1, UART_INT_FLAG_TXC) == RESET);
    }

    /* Runing led toggle */
    if(ledTime < GetTick())
    {
      ledTime = GetTick() + 1000;
      EVAL_LedToggle(LED1);
    }
  }
}

/* retarget the C library printf function to the usart */
int fputc(int ch, FILE *f)
{
  UART_SendData(EVAL_COM0, (uint8_t)ch);

  while(RESET == UART_GetIntFlagStatus(EVAL_COM0, UART_INT_FLAG_TXE));

  return ch;
}




/**
  * @}
  */

/**
  * @}
  */

/**
  * @}
  */

/************************ (C) COPYRIGHT DP *****END OF FILE ***************************/
